#include "impfunc.h"

bool ProcessImportTable(FILEINFO* fileinfo)
{
    IMAGE_DATA_DIRECTORY* pdd=fileinfo->pnth->OptionalHeader.DataDirectory;
    unsigned int size=pdd[1].Size;
    unsigned int va=pdd[1].VirtualAddress;
    if(!size or !va)
        return false;
    unsigned int offset=RVAToOffset(fileinfo->pnth, va);
    if(offset==0xFFFFFFFF)
        return false;
    IMAGE_IMPORT_DESCRIPTOR* pid=(IMAGE_IMPORT_DESCRIPTOR*)(fileinfo->lpFile+offset);
    int count=0;
    while(pid[count].OriginalFirstThunk)
        count++;
    if(!count)
        return false;
    IMPORT_DIR* impdir=(IMPORT_DIR*)malloc(sizeof(IMPORT_DIR));
    memset(impdir, 0, sizeof(IMPORT_DIR));
    impdir->num_descriptors=count;
    impdir->size=size;
    impdir->offset=offset;
    impdir->va=va;
    impdir->descriptors=(IMPORT_DESCRIPTOR*)malloc(sizeof(IMPORT_DESCRIPTOR)*count);
    memset(impdir->descriptors, 0, sizeof(IMPORT_DESCRIPTOR)*count);
    for(int i=0; i<count; i++)
    {
        char* name=(char*)(fileinfo->lpFile+RVAToOffset(fileinfo->pnth, pid[i].Name));
        unsigned int ofirstthunk=pid[i].OriginalFirstThunk;
        unsigned int firstthunk=pid[i].FirstThunk;
        unsigned int numthunks=0;
        unsigned int* thunks=(unsigned int*)(fileinfo->lpFile+RVAToOffset(fileinfo->pnth, ofirstthunk));
        while(thunks[numthunks])
            numthunks++;
        impdir->descriptors[i].dll_name=name;
        impdir->descriptors[i].first_thunk=firstthunk;
        impdir->descriptors[i].original_first_thunk=ofirstthunk;
        impdir->descriptors[i].num_thunks=numthunks;
        impdir->descriptors[i].thunks=(IMPORT_THUNK*)malloc(sizeof(IMPORT_THUNK)*numthunks);
        memset(impdir->descriptors[i].thunks, 0, sizeof(IMPORT_THUNK)*numthunks);
        for(unsigned int j=0; j<numthunks; j++)
        {
            unsigned int firstthunk=thunks[j];
            if(firstthunk&IMAGE_ORDINAL_FLAG)
            {
                impdir->descriptors[i].thunks[j].no_name=true;
                impdir->descriptors[i].thunks[j].name=(char*)(firstthunk^IMAGE_ORDINAL_FLAG);
            }
            else
            {
                unsigned int rawthunk=RVAToOffset(fileinfo->pnth, firstthunk);
                unsigned short* hint=(unsigned short*)(fileinfo->lpFile+rawthunk);
                char* api=(char*)(fileinfo->lpFile+rawthunk+sizeof(unsigned short));
                impdir->descriptors[i].thunks[j].name=api;
                //AddLogMessage("%s", api);
                impdir->descriptors[i].thunks[j].hint=*hint;
            }
        }
    }
    fileinfo->impdir=impdir;
    return true;
}

void FreeImportDirStruct(IMPORT_DIR* impdir)
{
    for(unsigned int i=0; i<impdir->num_descriptors; i++)
    {
        for(unsigned int j=0; j<impdir->descriptors[i].num_thunks; j++)
            free(&impdir->descriptors[i].thunks[j]);
        free(&impdir->descriptors[i]);
    }
    free(impdir);
}
