#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <ctype.h>
#include <richedit.h>
#include "resource.h"
#include "pefunc.h"
#include "multiasmlib/multiasmlib.h"

HINSTANCE hInst;

#define return_text_size 25600

const char* label_text=0;
char return_text[return_text_size];
char szFileName[256]="";
FILEINFO* fileinfo=0;
unsigned int iat_vp=0;

BOOL CALLBACK DlgArgument(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch(uMsg)
    {
    case WM_INITDIALOG:
    {
        if(label_text)
            SetDlgItemTextA(hwndDlg, IDC_STC_LABEL, label_text);
        memset(return_text, 0, return_text_size);
        EnableWindow(GetDlgItem(hwndDlg, IDC_BTN_OK), 0);
    }
    return TRUE;

    case WM_CLOSE:
    {
        EndDialog(hwndDlg, 0);
    }
    return TRUE;

    case WM_COMMAND:
    {
        switch(LOWORD(wParam))
        {
        case IDC_EDT_TEXT:
        {
            bool enable=false;
            if(GetWindowTextLengthA(GetDlgItem(hwndDlg, LOWORD(wParam))))
                enable=true;
            EnableWindow(GetDlgItem(hwndDlg, IDC_BTN_OK), enable);
        }
        return TRUE;

        case IDC_BTN_OK:
        {
            GetDlgItemTextA(hwndDlg, IDC_EDT_TEXT, return_text, return_text_size);
            SendMessageA(hwndDlg, WM_CLOSE, 0, 0);
        }
        return TRUE;
        }
    }
    return TRUE;
    }
    return FALSE;
}

void InsertText(HWND richedit, const char* text)
{
    SendMessage(richedit, EM_REPLACESEL, TRUE, (LPARAM)text);
    SetFocus(richedit);
}

unsigned int FindInImportTable(FILEINFO* pFileInfo, const char* dll, const char* api)
{
    IMPORT_DIR* impdir=pFileInfo->impdir;
    if(!impdir)
        return 0;
    for(unsigned int i=0; i<impdir->num_descriptors; i++)
        if(!strcasecmp(dll, impdir->descriptors[i].dll_name))
            for(unsigned int j=0; j<impdir->descriptors[i].num_thunks; j++)
                if(!impdir->descriptors[i].thunks[j].no_name and !strcmp(api, impdir->descriptors[i].thunks[j].name))
                    return impdir->descriptors[i].first_thunk+sizeof(unsigned int)*j;
    return 0;
}

void CopyToClipboard(const char* text)
{
    HGLOBAL hText;
    char *pText;
    int len=strlen(text);

    hText = GlobalAlloc(GMEM_DDESHARE | GMEM_MOVEABLE, len+1);
    pText = (char*)GlobalLock(hText);
    strcpy(pText, text);

    OpenClipboard(0);
    EmptyClipboard();
    if(len)
        if(!SetClipboardData(CF_OEMTEXT, hText))
            MessageBeep(MB_ICONERROR);
    CloseClipboard();
}

bool BrowseFileSave(HWND owner, const char* filter, const char* defext, char* filename, int filename_size, const char* init_dir)
{
    OPENFILENAME ofstruct;
    memset(&ofstruct, 0, sizeof(ofstruct));
    ofstruct.lStructSize=sizeof(ofstruct);
    ofstruct.hwndOwner=owner;
    ofstruct.hInstance=hInst;
    ofstruct.lpstrFilter=filter;
    ofstruct.lpstrFile=filename;
    ofstruct.nMaxFile=filename_size;
    ofstruct.lpstrInitialDir=init_dir;
    ofstruct.lpstrDefExt=defext;
    ofstruct.Flags=OFN_EXTENSIONDIFFERENT|OFN_HIDEREADONLY|OFN_NONETWORKBUTTON|OFN_OVERWRITEPROMPT;
    return GetSaveFileNameA(&ofstruct);
}

BOOL CALLBACK DlgMain(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch(uMsg)
    {
    case WM_INITDIALOG:
    {
        InitLog(hwndDlg);
        EnableWindow(GetDlgItem(hwndDlg, IDC_BTN_LOAD), 0);
        EnableWindow(GetDlgItem(hwndDlg, IDC_BTN_ASSEMBLE), 0);
        EnableWindow(GetDlgItem(hwndDlg, IDC_BTN_COPYCODE), 0);
    }
    return TRUE;

    case WM_CLOSE:
    {
        EndDialog(hwndDlg, 0);
    }
    return TRUE;

    case WM_DROPFILES:
    {
        int len=DragQueryFileA((HDROP)wParam, 0, szFileName, 256);
        while(szFileName[len]!='\\')
            len--;
        strcpy(return_text, szFileName+len+1);
        SetDlgItemTextA(hwndDlg, IDC_STC_FILE, return_text);

        if(fileinfo)
            FreeFileInfoStruct(fileinfo);
        fileinfo=(FILEINFO*)malloc(sizeof(FILEINFO));
        memset(fileinfo, 0, sizeof(FILEINFO));
        strcpy(fileinfo->FilePath, szFileName);
        SendMessageA(hwndDlg, WM_COMMAND, IDC_BTN_LOAD, 0);
        EnableWindow(GetDlgItem(hwndDlg, IDC_BTN_LOAD), 1);
        EnableWindow(GetDlgItem(hwndDlg, IDC_BTN_ASSEMBLE), 1);
        EnableWindow(GetDlgItem(hwndDlg, IDC_BTN_COPYCODE), 1);
    }
    return TRUE;

    case WM_COMMAND:
    {
        switch(LOWORD(wParam))
        {
        case IDC_BTN_COPYCODE:
        {
            char sApirva[10]="";
            GetDlgItemTextA(hwndDlg, IDC_EDT_APIRVA, sApirva, 10);
            char sEmpty[10]="";
            GetDlgItemTextA(hwndDlg, IDC_EDT_EMPTY, sEmpty, 10);
            HWND richedit=GetDlgItem(hwndDlg, IDC_EDT_CODE);
            GETTEXTLENGTHEX lenex= {0};
            lenex.flags=GTL_CLOSE;
            lenex.codepage=CP_ACP;
            unsigned int sCode_size=SendMessageA(richedit, EM_GETTEXTLENGTHEX, (WPARAM)&lenex, 0)+4096;
            char* sCode=(char*)malloc(sCode_size);
            memset(sCode, 0, sCode_size);
            GetDlgItemTextA(hwndDlg, IDC_EDT_CODE, sCode, sCode_size);
            unsigned int apirva=0;
            sscanf(sApirva, "%x", &apirva);
            unsigned int emptyspace=0;
            sscanf(sEmpty, "%x", &emptyspace);
            unsigned int sub_const=emptyspace-fileinfo->dwImageBase+6;
            unsigned int oep=fileinfo->dwEntrypoint+fileinfo->dwImageBase;
            char* sFinalCode=(char*)malloc(sCode_size);
            memset(sFinalCode, 0, sCode_size);
            GetDlgItemTextA(hwndDlg, IDC_EDT_CODE, sCode, sCode_size);
            sprintf(sFinalCode,
                    "pushad\r\ncall @f\r\n@@:\r\npop ebp\r\nsub ebp,0%X\r\nmov ebx,dword ptr ds:[ebp+0%X]\r\nmov esi,dword ptr ds:[ebp+0%X]\r\ncall @f\r\n@vpex_addr:\r\nmov eax,0FFFFFFFF\r\nret\r\n@@:\r\npop eax\r\nmov dword ptr ds:[eax+1],esi\r\ncall @f\r\n\"\\x00\\x00\\x00\\x00\"\r\n@@:\r\npush 40\r\npush 50\r\npush esi\r\ncall ebx\r\ncall @skip\r\n@original_bytes:\r\ncall @f\r\n\"\\x90\\x90\\x90\\x90\\x90\"\r\n@@:\r\njmp short @hook_back\r\n@skip:\r\npop edi\r\nadd edi,5\r\nmov ecx,5\r\nrep movs byte ptr es:[edi],byte ptr ds:[esi]\r\nsub esi,5\r\nmov byte ptr ds:[esi],0E9\r\ncall @f\r\n@hook_VirtualProtectEx:\r\npush edi\r\npush esi\r\npush ecx\r\njmp short @original_bytes\r\n@hook_back:\r\npop esi\r\ncall @vpex_addr\r\nmov edi,eax\r\nmov ecx,5\r\nrep movs byte ptr es:[edi],byte ptr ds:[esi]\r\npushad\r\npushfd\r\n%s\r\npopfd\r\npopad\r\npop ecx\r\npop esi\r\npop edi\r\njmp eax\r\n@@:\r\npop eax\r\nsub eax,5\r\nsub eax,esi\r\nmov dword ptr ds:[esi+1],eax\r\npopad\r\njmp 0%X",
                    sub_const, iat_vp, apirva, sCode, oep);
            CopyToClipboard(sFinalCode);
            MessageBeep(MB_ICONINFORMATION);
        }
        return TRUE;

        case IDC_BTN_ASSEMBLE:
        {
            char sApirva[10]="";
            GetDlgItemTextA(hwndDlg, IDC_EDT_APIRVA, sApirva, 10);
            char sEmpty[10]="";
            GetDlgItemTextA(hwndDlg, IDC_EDT_EMPTY, sEmpty, 10);
            HWND richedit=GetDlgItem(hwndDlg, IDC_EDT_CODE);
            GETTEXTLENGTHEX lenex= {0};
            lenex.flags=GTL_CLOSE;
            lenex.codepage=CP_ACP;
            unsigned int sCode_size=SendMessageA(richedit, EM_GETTEXTLENGTHEX, (WPARAM)&lenex, 0)+4096;
            char* sCode=(char*)malloc(sCode_size);
            memset(sCode, 0, sCode_size);
            GetDlgItemTextA(hwndDlg, IDC_EDT_CODE, sCode, sCode_size);
            char sError[1024]="";
            int errorOffset=0;
            DWORD asmSize=0;
            BYTE* code=MUAAssemble(sCode, 0x401000, &asmSize, 0x400000, &errorOffset, sError);
            if(!code)
            {
                int totalsub=0;
                for(int i=0; i<errorOffset; i++)
                    if(sCode[i]=='\r' or sCode[i]=='\n')
                        totalsub++;
                MessageBoxA(hwndDlg, sError, "Error!", MB_ICONERROR);
                CHARRANGE ch= {0};
                ch.cpMax=ch.cpMin=errorOffset-totalsub;
                SendMessageA(richedit, EM_EXSETSEL, 0, (LPARAM)&ch);
                SetFocus(richedit);
                return TRUE;
            }
            MUAAssembleFree(code);
            unsigned int apirva=0;
            sscanf(sApirva, "%x", &apirva);
            unsigned int emptyspace=0;
            sscanf(sEmpty, "%x", &emptyspace);
            unsigned int sub_const=emptyspace-fileinfo->dwImageBase+6;
            unsigned int oep=fileinfo->dwEntrypoint+fileinfo->dwImageBase;
            char* sFinalCode=(char*)malloc(sCode_size);
            memset(sFinalCode, 0, sCode_size);
            GetDlgItemTextA(hwndDlg, IDC_EDT_CODE, sCode, sCode_size);
            sprintf(sFinalCode,
                    "pushad\r\ncall @f\r\n@@:\r\npop ebp\r\nsub ebp,0%X\r\nmov ebx,dword ptr ds:[ebp+0%X]\r\nmov esi,dword ptr ds:[ebp+0%X]\r\ncall @f\r\n@vpex_addr:\r\nmov eax,0FFFFFFFF\r\nret\r\n@@:\r\npop eax\r\nmov dword ptr ds:[eax+1],esi\r\ncall @f\r\n\"\\x00\\x00\\x00\\x00\"\r\n@@:\r\npush 40\r\npush 50\r\npush esi\r\ncall ebx\r\ncall @skip\r\n@original_bytes:\r\ncall @f\r\n\"\\x90\\x90\\x90\\x90\\x90\"\r\n@@:\r\njmp short @hook_back\r\n@skip:\r\npop edi\r\nadd edi,5\r\nmov ecx,5\r\nrep movs byte ptr es:[edi],byte ptr ds:[esi]\r\nsub esi,5\r\nmov byte ptr ds:[esi],0E9\r\ncall @f\r\n@hook_VirtualProtectEx:\r\npush edi\r\npush esi\r\npush ecx\r\njmp short @original_bytes\r\n@hook_back:\r\npop esi\r\ncall @vpex_addr\r\nmov edi,eax\r\nmov ecx,5\r\nrep movs byte ptr es:[edi],byte ptr ds:[esi]\r\npushad\r\npushfd\r\n%s\r\npopfd\r\npopad\r\npop ecx\r\npop esi\r\npop edi\r\njmp eax\r\n@@:\r\npop eax\r\nsub eax,5\r\nsub eax,esi\r\nmov dword ptr ds:[esi+1],eax\r\npopad\r\njmp 0%X",
                    sub_const, iat_vp, apirva, sCode, oep);
            code=MUAAssemble(sFinalCode, emptyspace, &asmSize, fileinfo->dwImageBase, &errorOffset, sError);
            if(!code)
            {
                MessageBoxA(hwndDlg, "Unknown Assemble Error!", "Error!", MB_ICONERROR);
                return TRUE;
            }
            char file[256]="";
            char filedir[256]="";
            strcpy(filedir, fileinfo->FilePath);
            int len=strlen(filedir);
            while(filedir[len]!='\\')
                len--;
            filedir[len]=0;
            BrowseFileSave(hwndDlg, "Executables (*.exe)\0*.exe\0\0", "exe", file, 256, filedir);
            if(!*file)
            {
                MUAAssembleFree(code);
                return TRUE;
            }
            CopyFileA(fileinfo->FilePath, file, 0);
            HANDLE hFile=CreateFileA(file, GENERIC_WRITE, 0, 0, OPEN_EXISTING, 0, 0);
            if(hFile==INVALID_HANDLE_VALUE)
            {
                MessageBoxA(hwndDlg, "Could not open file with read access!", "Error", MB_ICONERROR);
                DeleteFileA(file);
                MUAAssembleFree(code);
                return TRUE;
            }
            if(SetFilePointer(hFile, RVAToOffset(fileinfo->pnth, emptyspace-fileinfo->dwImageBase), 0, FILE_BEGIN)==0xFFFFFFFF)
            {
                MessageBoxA(hwndDlg, "Could not set file pointer (invalid empty space)", "Error", MB_ICONERROR);
                CloseHandle(hFile);
                DeleteFileA(file);
                MUAAssembleFree(code);
                return TRUE;
            }
            DWORD written=0;
            if(!WriteFile(hFile, code, asmSize, &written, 0))
            {
                MessageBoxA(hwndDlg, "Could not write file", "Error", MB_ICONERROR);
                CloseHandle(hFile);
                DeleteFileA(file);
                MUAAssembleFree(code);
                return TRUE;
            }
            unsigned char* a=(unsigned char*)(&fileinfo->pnth->OptionalHeader.AddressOfEntryPoint);
            unsigned char* b=fileinfo->lpFile;
            unsigned int offset=a-b;
            unsigned int new_data=emptyspace-fileinfo->dwImageBase;
            SetFilePointer(hFile, offset, 0, FILE_BEGIN);
            written=0;
            if(!WriteFile(hFile, &new_data, sizeof(unsigned int), &written, 0))
            {
                MessageBoxA(hwndDlg, "Could not write file", "Error", MB_ICONERROR);
                CloseHandle(hFile);
                DeleteFileA(file);
                MUAAssembleFree(code);
                return TRUE;
            }
            CloseHandle(hFile);
            MessageBoxA(hwndDlg, "File written!", "Done!", MB_ICONINFORMATION);
            MUAAssembleFree(code);
        }
        return TRUE;

        case IDC_BTN_LOAD:
        {
            if(!ReadInFile(fileinfo))
            {
                FreeFileInfoStruct(fileinfo);
                fileinfo=0;
                AddLogMessage("Could not read file!");
                return TRUE;
            }
            if(!FillFileData(fileinfo))
            {
                FreeFileInfoStruct(fileinfo);
                fileinfo=0;
                AddLogMessage("Failed to fill file data!");
                return TRUE;
            }
            if(!ProcessImportTable(fileinfo))
            {
                FreeFileInfoStruct(fileinfo);
                fileinfo=0;
                AddLogMessage("Failed to process import table!");
                return TRUE;
            }
            iat_vp=FindInImportTable(fileinfo, "kernel32.dll", "VirtualProtect");
            if(!iat_vp)
            {
                FreeFileInfoStruct(fileinfo);
                fileinfo=0;
                AddLogMessage("Could not find VirtualProtect in the import table!");
                return TRUE;
            }
            char temp[10]="";
            sprintf(temp, "%.8X", fileinfo->dwEntrypoint+fileinfo->dwImageBase);
            SetDlgItemTextA(hwndDlg, IDC_EDT_OEP, temp);
            char dll[256]="";
            char api[256]="";
            if(!GetDlgItemTextA(hwndDlg, IDC_EDT_LIB, dll, 256) or !GetDlgItemTextA(hwndDlg, IDC_EDT_API, api, 256))
                return TRUE;
            unsigned int rva=FindInImportTable(fileinfo, dll, api);
            if(!rva)
            {
                AddLogMessage("Could not find API %s in DLL %s", api, dll);
                return TRUE;
            }
            sprintf(temp, "%.8X", rva);
            SetDlgItemTextA(hwndDlg, IDC_EDT_APIRVA, temp);
        }
        return TRUE;

        case IDC_BTN_ABOUT:
        {
            MessageBoxA(hwndDlg, "A program demonstrating multiasmlib.dll\n\nCreated by: Mr. eXoDia", "About :)", MB_ICONINFORMATION);
        }
        return TRUE;

        case IDC_BTN_VARIABLE:
        {
            label_text="Variable Name:";
            DialogBoxParamA(hInst, MAKEINTRESOURCE(DLG_ARGUMENT), hwndDlg, DlgArgument, 0);
            char format_text[return_text_size]="";
            if(!strlen(return_text))
            {
                MessageBeep(MB_ICONERROR);
                return TRUE;
            }
            srand(GetTickCount());
            unsigned int rnd=rand()%65536;
            sprintf(format_text, "call @%d_%s\r\n@%s:\r\n\r\n@%d_%s:\r\npop ebp", rnd, return_text, return_text, rnd, return_text);
            HWND richedit=GetDlgItem(hwndDlg, IDC_EDT_CODE);
            InsertText(richedit, format_text);
            CHARRANGE ch= {0};
            SendMessageA(richedit, EM_EXGETSEL, 0, (LPARAM)&ch);
            ch.cpMax-=12;
            ch.cpMin-=12;
            SendMessageA(richedit, EM_EXSETSEL, 0, (LPARAM)&ch);
        }
        return TRUE;

        case IDC_BTN_BYTES:
        {
            label_text="Bytes (hex):";
            DialogBoxParamA(hInst, MAKEINTRESOURCE(DLG_ARGUMENT), hwndDlg, DlgArgument, 0);
            char format_text[return_text_size]="";
            int len=strlen(return_text);
            for(int i=0,j=0; i<len; i++)
                if(isxdigit(return_text[i]))
                    j+=sprintf(format_text+j, "%c", toupper(return_text[i]));
            len=strlen(format_text);
            if(!len)
            {
                MessageBeep(MB_ICONERROR);
                return TRUE;
            }
            if(len%2)
            {
                strcat(format_text, "0");
                len++;
            }
            strcpy(return_text, format_text);
            strcpy(format_text, "\"");
            for(int i=0,j=1; i<len; i+=2)
                j+=sprintf(format_text+j, "\\x%c%c", return_text[i], return_text[i+1]);
            strcat(format_text, "\"");
            InsertText(GetDlgItem(hwndDlg, IDC_EDT_CODE), format_text);
        }
        return TRUE;

        case IDC_BTN_ARG:
        {
            label_text="Argument Number (dec):";
            DialogBoxParamA(hInst, MAKEINTRESOURCE(DLG_ARGUMENT), hwndDlg, DlgArgument, 0);
            char format_text[return_text_size]="";
            int len=strlen(return_text);
            for(int i=0,j=0; i<len; i++)
                if(isdigit(return_text[i]))
                    j+=sprintf(format_text+j, "%c", return_text[i]);
            int arg_number=0;
            sscanf(format_text, "%d", &arg_number);
            sprintf(return_text, "0%X", 0x30+(arg_number*4));
            bool noprep=false;
            if(isdigit(return_text[1]))
                noprep=true;
            sprintf(format_text, "dword ptr ds:[esp+%s]", return_text+noprep);
            InsertText(GetDlgItem(hwndDlg, IDC_EDT_CODE), format_text);
        }
        return TRUE;

        case IDC_BTN_RETURN:
        {
            InsertText(GetDlgItem(hwndDlg, IDC_EDT_CODE), "dword ptr ds:[esp+30]");
        }
        return TRUE;
        }
    }
    return TRUE;
    }
    return FALSE;
}

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
    hInst=hInstance;
    InitCommonControls();
    LoadLibrary("riched20.dll");
    memset(return_text, 0, return_text_size);
    return DialogBox(hInst, MAKEINTRESOURCE(DLG_MAIN), NULL, (DLGPROC)DlgMain);
}
